//{{NO_DEPENDENCIES}}
// Microsoft Developer Studio generated include file.
// Used by comskip.rc
//
#define ID_LEFT_BUTTON                  1
#define ID_LEFT_ARROW                   2
#define ID_RIGHT_ARROW                  3
#define ID_RIGHT_BUTTON                 4
#define ID_TRACKBAR                     5
#define ID_ADD                          100
#define ID_DEL                          101
#define IDD_ABOUT                       102
#define ID_ADD_AUTO                     102
#define ID_UP                           102
#define IDM_ABOUT                       103
#define ID_DELALL                       103
#define IDM_EXIT                        104
#define ID_DOWN                         104
#define IDI_SMALL                       105
#define IDC_GUI                         106
#define IDI_MOVIE                       133
#define IDR_ACCELERATOR                 134
#define IDD_INFO                        135
#define IDD_FILELIST                    136
#define IDD_DELAY                       137
#define IDD_LUMINANCE                   138
#define IDD_NORM                        139
#define IDD_CLIPRESIZE                  140
#define IDD_SET_PIDS                    141
#define IDD_DETECT_PIDS                 142
#define IDD_SPEED                       143
#define IDC_CODED_NUMBER                1016
#define IDC_PLAYBACK_NUMBER             1017
#define IDC_FRAME_RATE                  1018
#define IDC_VOB_ID                      1019
#define IDC_CELL_ID                     1020
#define IDC_VIDEO_TYPE                  1021
#define IDC_FRAME_TYPE                  1022
#define IDC_ASPECT_RATIO                1023
#define IDC_REMAIN                      1024
#define IDC_ELAPSED                     1025
#define IDC_FILE_SIZE                   1026
#define IDC_FRAME_SIZE                  1026
#define IDC_FILE                        1027
#define IDC_PROFILE                     1027
#define IDC_FPS                         1028
#define IDC_COLORIMETRY                 1029
#define IDC_TIMESTAMP                   1030
#define IDC_LIST                        1031
#define IDC_FRAME_STRUCTURE             1031
#define IDC_INFO                        1032
#define IDC_BITRATE                     1033
#define IDC_DELAY                       1034
#define IDC_FIELD_ORDER                 1034
#define IDC_OFFSET_SLIDER               1035
#define IDC_BITRATE_AVG                 1035
#define IDC_GAMMA_SLIDER                1036
#define IDC_LUM_CHECK                   1037
#define IDC_OFFSET                      1038
#define IDC_GAIN                        1039
#define IDC_NORM_SLIDER                 1040
#define IDC_NORM_CHECK                  1041
#define IDC_NORM                        1042
#define IDC_TOP_SLIDER                  1043
#define IDC_BOTTOM_SLIDER               1044
#define IDC_TOP                         1045
#define IDC_BOTTOM                      1046
#define IDC_RIGHT_SLIDER                1047
#define IDC_LEFT_SLIDER                 1048
#define IDC_RIGHT                       1049
#define IDC_LEFT                        1050
#define IDC_CLIPRESIZE_CHECK            1051
#define IDC_HEIGHT_SLIDER               1052
#define IDC_WIDTH_SLIDER                1053
#define IDC_VERSION                     1054
#define IDC_HEIGHT                      1055
#define IDC_WIDTH                       1056
#define IDC_LBH_SLIDER                  1057
#define IDC_LBW_SLIDER                  1058
#define IDC_LBH                         1059
#define IDC_LBW                         1060
#define IDC_VIDEO_PID                   1061
#define IDC_AUDIO_PID                   1062
#define IDC_PIDS_OK                     1063
#define IDC_PIDS_CANCEL                 1064
#define IDC_PID_LISTBOX                 1064
#define IDC_SET_AUDIO                   1065
#define IDC_SET_VIDEO                   1066
#define IDC_AUDIO_TYPE                  1067
#define IDC_SET_DONE                    1067
#define IDC_AUDIO_TYPE2                 1068
#define IDC_AUDIO_TYPE3                 1069
#define IDC_SPEED_SLIDER                1069
#define IDC_AUDIO_TYPE4                 1070
#define IDC_AUDIO_TYPE5                 1071
#define IDC_AUDIO_TYPE6                 1072
#define IDC_AUDIO_TYPE7                 1073
#define IDC_AUDIO_TYPE8                 1074
#define IDM_OPEN                        32771
#define IDM_SAVE_D2V                    32772
#define IDM_SAVE_AC3                    32773
#define IDM_LOAD_D2V                    32774
#define IDM_PREVIEW                     32775
#define IDM_PAUSE                       32777
#define IDM_STOP                        32778
#define IDM_DISPLAY                     32779
#define IDM_PP_HIGH                     32780
#define IDM_PP_NORMAL                   32781
#define IDM_PP_LOW                      32782
#define IDM_STORE_RGB24                 32783
#define IDM_STORE_YUY2                  32784
#define IDM_FO_FILM                     32785
#define IDM_INFO                        32786
#define IDM_DIRECTDRAW                  32787
#define IDM_VFAPI                       32788
#define IDM_IDCT_MMX                    32789
#define IDM_IDCT_FPU                    32790
#define IDM_IDCT_REF                    32791
#define IDM_MMX                         32792
#define IDM_SSEMMX                      32794
#define IDM_HOMEPAGE                    32795
#define IDM_PROCESS_WAV                 32796
#define IDM_SRC_NONE                    32797
#define IDM_SRC_LOW                     32798
#define IDM_SRC_MID                     32799
#define IDM_SRC_HIGH                    32800
#define IDM_SRC_UHIGH                   32801
#define IDM_NORM                        32802
#define IDM_TRACK_1                     32804
#define IDM_TRACK_2                     32805
#define IDM_TRACK_3                     32806
#define IDM_TRACK_4                     32807
#define IDM_TRACK_5                     32808
#define IDM_TRACK_6                     32809
#define IDM_TRACK_7                     32810
#define IDM_TRACK_8                     32811
#define IDM_TVSCALE                     32821
#define IDM_PCSCALE                     32822
#define IDM_BMP                         32823
#define IDM_LUMINANCE                   32824
#define IDM_DSDOWN                      32825
#define IDM_FO_NONE                     32826
#define IDM_DRC_NONE                    32828
#define IDM_DRC_LIGHT                   32829
#define IDM_DRC_NORMAL                  32830
#define IDM_DRC_HEAVY                   32831
#define IDM_CLIPRESIZE                  32834
#define IDM_PRESCALE                    32835
#define IDM_IDCT_SSEMMX                 32836
#define IDM_3DNOW                       32837
#define IDM_SSEFPU                      32838
#define IDM_KEY_OP                      32842
#define IDM_KEY_INPUT                   32843
#define IDM_KEY_OFF                     32844
#define IDM_SSE2                        32845
#define IDM_IDCT_SSE2MMX                32846
#define IDM_DEMUX                       32850
#define IDM_DEMUXALL                    32851
#define IDM_DECODE                      32852
#define IDM_AUDIO_NONE                  32853
#define IDM_OPEN_AUTO                   32860
#define IDM_PLAY                        32862
#define IDM_JACKEI                      32863
#define IDM_NEURON2                     32864
#define IDM_FO_RAW                      32865
#define IDM_SET_PIDS                    32866
#define IDM_DETECT_PIDS                 32867
#define IDM_DETECT_PIDS_RAW             32868
#define IDM_SAVE_D2V_AND_DEMUX          32870
#define IDM_PARSE_D2V                   32871
#define IDM_FIX_D2V                     32872
#define IDM_SPEED                       32873
#define IDM_IDCT_SKAL                   32874
#define IDM_IDCT_SIMPLE                 32875
#define IDM_LOG_QUANTS                  32877
#define IDM_SPEED_SUPER_SLOW            32878
#define IDM_SPEED_SLOW                  32879
#define IDM_SPEED_NORMAL                32880
#define IDM_SPEED_FAST                  32881
#define IDM_SPEED_MAXIMUM               32882
#define IDM_FORCE_OPEN                  32883
#define IDM_SPEED_SINGLE_STEP           32884
#define IDM_DGINDEX_MANUAL              32885
#define IDM_DGDECODE_MANUAL             32886
#define IDM_QUICK_START                 32887
#define IDM_CORRECT_FIELD_ORDER         32888
#define IDC_STATIC                      -1

// Next default values for new objects
// 
#ifdef APSTUDIO_INVOKED
#ifndef APSTUDIO_READONLY_SYMBOLS
#define _APS_NEXT_RESOURCE_VALUE        144
#define _APS_NEXT_COMMAND_VALUE         32889
#define _APS_NEXT_CONTROL_VALUE         1075
#define _APS_NEXT_SYMED_VALUE           107
#endif
#endif
